@@ -35,7 +35,7 @@ def download_ref_data(params):
 
 class SCSiliconParams:
 
-    def __init__(self, out_dir='./', data_dir='./data', ref='hg19', chrom='chr22', layout='PE', coverage=5, isize=260, threads=1, verbose=1):
+    def __init__(self, out_dir='./', data_dir='./data', ref='hg19', chrom='chr22', layout='SE', coverage=5, isize=260, threads=1, verbose=1):
         self.out_dir = out_dir
         self.data_dir = data_dir
         self.ref = ref
@@ -148,11 +148,11 @@ class SCSiliconParams:
 
 
 class SNPSimulator:
-    def __init__(self, sample_no=1, snp_no=1000, verbose=1):
-        self.sample_no = sample_no
+    def __init__(self, cell_no=1, snp_no=1000, verbose=1):
+        self.cell_no = cell_no
         self.snp_no = snp_no
         self._check_params()
-        self.samples = dict.fromkeys(['sample' + str(i+1) for i in range(self.sample_no)])
+        self.samples = dict.fromkeys(['cell' + str(i+1) for i in range(self.cell_no)])
         for sample in self.samples:
             self.samples[sample] = {}
         tasklogger.set_level(verbose)
@@ -169,8 +169,8 @@ class SNPSimulator:
         ------
         ValueError : unacceptable choice of parameters
         """
-        utils.check_int(sample_no=self.sample_no)
-        utils.check_positive(sample_no=self.sample_no)
+        utils.check_int(cell_no=self.cell_no)
+        utils.check_positive(cell_no=self.cell_no)
         utils.check_int(snp_no=self.snp_no)
         utils.check_positive(snp_no=self.snp_no)
 
@@ -183,7 +183,7 @@ class SNPSimulator:
         Parameters
         ----------
 
-        sample_no : int, optional, default: 1
+        cell_no : int, optional, default: 1
             The sample number for this simulation
         
         snp_no : int, optional, default: 1000
@@ -198,9 +198,9 @@ class SNPSimulator:
         """
 
         # parameters
-        if 'sample_no' in params and params['sample_no'] != self.sample_no:
-            self.sample_no = params['sample_no']
-            del params['sample_no']
+        if 'cell_no' in params and params['cell_no'] != self.cell_no:
+            self.cell_no = params['cell_no']
+            del params['cell_no']
         if 'snp_no' in params and params['snp_no'] != self.snp_no:
             self.snp_no = params['snp_no']
             del params['snp_no']
@@ -270,8 +270,8 @@ class SNPSimulator:
         self._generate_reads_for_snp(params)
 
 class CNVSimulator:
-    def __init__(self, sample_no=1, bin_len=500000, seg_no=10, cluster_no=1, normal_frac=0.4, noise_frac=0.1, verbose=1):
-        self.sample_no = sample_no
+    def __init__(self, cell_no=1, bin_len=500000, seg_no=10, cluster_no=1, normal_frac=0.4, noise_frac=0.1, verbose=1):
+        self.cell_no = cell_no
         self.bin_len = bin_len
         self.seg_no = seg_no
         self.cluster_no = cluster_no
@@ -282,7 +282,7 @@ class CNVSimulator:
         self.segments = None
         self.clusters = None
         self._check_params()
-        self.samples = dict.fromkeys(['sample' + str(i+1) for i in range(self.sample_no)])
+        self.samples = dict.fromkeys(['cell' + str(i+1) for i in range(self.cell_no)])
         for sample in self.samples:
             self.samples[sample] = {}
         tasklogger.set_level(verbose)
@@ -299,8 +299,8 @@ class CNVSimulator:
         ------
         ValueError : unacceptable choice of parameters
         """
-        utils.check_int(sample_no=self.sample_no)
-        utils.check_positive(sample_no=self.sample_no)
+        utils.check_int(cell_no=self.cell_no)
+        utils.check_positive(cell_no=self.cell_no)
         utils.check_int(bin_len=self.bin_len)
         utils.check_positive(bin_len=self.bin_len)
         utils.check_int(seg_no=self.seg_no)
@@ -319,7 +319,7 @@ class CNVSimulator:
         Parameters
         ----------
 
-        sample_no : int, optional, default: 1
+        cell_no : int, optional, default: 1
             The sample number for this simulation
         
         bin_len : int, optional, default: 500000
@@ -346,9 +346,9 @@ class CNVSimulator:
         """
 
         # parameters
-        if 'sample_no' in params and params['sample_no'] != self.sample_no:
-            self.sample_no = params['sample_no']
-            del params['sample_no']
+        if 'cell_no' in params and params['cell_no'] != self.cell_no:
+            self.cell_no = params['cell_no']
+            del params['cell_no']
         if 'bin_len' in params and params['bin_len'] != self.bin_len:
             self.bin_len = params['bin_len']
             del params['bin_len']
@@ -428,10 +428,10 @@ class CNVSimulator:
 
     def _generate_cnv_matrix(self, params):
         ref = self._split_chr_to_bins(params)
-        columns = ['sample' + str(i+1) for i in range(self.sample_no)]
+        columns = ['cell' + str(i+1) for i in range(self.cell_no)]
         cnv = pd.DataFrame(columns=columns)
         segments = pd.DataFrame(columns=['Chromosome', 'Start_bin_no', 'End_bin_no'])
-        clusters_file = pd.DataFrame(columns=['Sample', 'Cluster'])
+        clusters_file = pd.DataFrame(columns=['Cell', 'Cluster'])
 
         if params.chrom != 'all':
             all_chroms = [params.chrom]
@@ -439,20 +439,21 @@ class CNVSimulator:
             all_chroms = sorted(np.unique(ref['Chromosome']), key=lambda x: int(x[3:]))
 
         # generate cell clusters
-        if self.cluster_no < 1 or self.cluster_no > self.sample_no:
+        if self.cluster_no < 1 or self.cluster_no > self.cell_no*(1-self.normal_frac) - 1:
             self.cluster_no = 1
         if self.cluster_no == 1:
             clusters  = []
         else:
-            clusters = np.sort(np.random.choice(range(1, self.sample_no-1), self.cluster_no - 1, replace=False))
-        clusters = [0] + list(clusters) + [self.sample_no]
+            clusters = np.sort(np.random.choice(range(int(self.cell_no*self.normal_frac)+1, self.cell_no-1), self.cluster_no - 2, replace=False))
+        clusters = [0] + [int(self.cell_no*self.normal_frac)] + list(clusters) + [self.cell_no]
+        print(clusters)
         cluster_count = 1
         for i in range(len(clusters) - 1):
             start = clusters[i]
             end = clusters[i+1]
             for j in range(start, end):
                 clusters_file = clusters_file.append([{
-                    'Sample': 'sample' + str(j+1),
+                    'Cell': 'cell' + str(j+1),
                     'Cluster': cluster_count,
                 }], ignore_index=True)
             cluster_count += 1
@@ -461,7 +462,7 @@ class CNVSimulator:
         for chrom in all_chroms:
             chrom_ref = ref[ref['Chromosome'] == chrom]
             total_bins = chrom_ref.shape[0]
-            sub_matrix = np.zeros((total_bins, self.sample_no),dtype=int)
+            sub_matrix = np.zeros((total_bins, self.cell_no),dtype=int)
             if self.seg_no < 1 or self.seg_no > total_bins:
                 self.seg_no = 10
             if self.seg_no == 1:
@@ -485,13 +486,13 @@ class CNVSimulator:
                     'Start_bin_no': start,
                     'End_bin_no': end
                 }], ignore_index=True)
-        cnv.iloc[:,0:max(0, round(self.sample_no*self.normal_frac))] = 2
+        cnv.iloc[:,0:max(0, round(self.cell_no*self.normal_frac))] = 2
 
         #add noise
         total_values = cnv.shape[0] * cnv.shape[1]
         noise = np.random.randint(0, total_values+1, size=round(total_values*self.noise_frac))
         for i in noise:
-            cnv.iat[int(i/cnv.shape[1]), i % cnv.shape[1]] = np.random.randint(1, 10)
+            cnv.iat[int((i-1)/cnv.shape[1]), (i-1) % cnv.shape[1]] = np.random.randint(1, 10)
 
         cnv = cnv.T
         cnv = cnv.astype('int32')
@@ -557,3 +558,302 @@ class CNVSimulator:
         self._generate_cnv_bed_file(params)
         # self._sim_fasta_for_cnv(params)
         self._generate_reads_for_cnv(params)
+
+class IndelSimulator:
+    def __init__(self, cell_no=1, in_no=100, del_no=100, verbose=1):
+        self.cell_no = cell_no
+        self.in_no = in_no
+        self.del_no = del_no       
+        self._check_params()
+        self.samples = dict.fromkeys(['cell' + str(i+1) for i in range(self.cell_no)])
+        for sample in self.samples:
+            self.samples[sample] = {}
+        tasklogger.set_level(verbose)
+
+
+    def _check_params(self):
+        """Check SCSilicon parameters
+
+        This allows us to fail early - otherwise certain unacceptable
+        parameter choices, such as threads='10.5', would only fail after
+        minutes of runtime.
+
+        Raises
+        ------
+        ValueError : unacceptable choice of parameters
+        """
+        utils.check_int(cell_no=self.cell_no)
+        utils.check_positive(cell_no=self.cell_no)
+        utils.check_int(in_no=self.in_no)
+        utils.check_positive(in_no=self.in_no)
+        utils.check_int(del_no=self.del_no)
+        utils.check_positive(del_no=self.del_no)
+
+    def set_params(self, **params):
+        """Set the parameters of SCSilicon.
+
+        Any parameters not given as named arguments will be left at their
+        current value.
+
+        Parameters
+        ----------
+
+        cell_no : int, optional, default: 1
+            The sample number for this simulation
+        
+        in_no : int, optional, default: 1000
+            The insertion number of each sample
+        
+        del_no : int, optional, default: 1000
+            The deletion number of each sample
+        
+        verbose : `int` or `boolean`, optional, default: 1
+            If `True` or `> 0`, print log messages
+
+        Returns
+        -------
+        self
+        """
+
+        # parameters
+        if 'cell_no' in params and params['cell_no'] != self.cell_no:
+            self.cell_no = params['cell_no']
+            del params['cell_no']
+        if 'in_no' in params and params['in_no'] != self.snp_no:
+            self.snp_no = params['in_no']
+            del params['in_no']
+        if 'del_no' in params and params['del_no'] != self.snp_no:
+            self.snp_no = params['del_no']
+            del params['del_no']
+        if 'verbose' in params:
+            self.verbose = params['verbose']
+            tasklogger.set_level(self.verbose)
+            del params['verbose']
+        self._check_params()
+        self.get_params()
+        return self
+
+    def get_params(self):
+        print(vars(self))
+
+    def _generate_random_seq(self):
+        chrs = ['a', 't', 'c', 'g']
+        seq_len = np.random.choice(range(4, 10))
+        seq = ''
+        for i in range(seq_len):
+            seq += np.random.choice(chrs)
+        return seq
+
+    def _generate_indel_file(self, params):
+        tasklogger.log_info('start generating indel profile file...')
+        chrom_sizes = pd.read_csv(params.chrom_size_file, header=None, sep='\t')
+        if params.chrom != 'all':
+            chrom = params.chrom
+            chrom_size = int(chrom_sizes[chrom_sizes[0]==chrom][1])
+        else:
+            chroms = {}
+            for index, row in chrom_sizes.iterrows():
+                [chrom, chrom_size] = row
+                chroms[chrom] = chrom_size
+
+        #out indel file
+        for sample in self.samples:
+            indel_file =os.path.join(params.out_dir, sample + '-indel.txt')
+            self.samples[sample]['indel_file'] = indel_file
+            
+            if params.chrom != 'all':
+                ins = np.sort(np.random.choice(range(1, chrom_size), self.in_no, replace=False))
+                dels = np.sort(np.random.choice(range(1, chrom_size), self.del_no, replace=False))
+                with open(indel_file, 'w') as output:
+                    for insert in ins:
+                        output.write('\t'.join(['i', chrom, str(insert), self._generate_random_seq(),np.random.choice(['homo', 'het'])])+'\n')
+                    for delete in dels:
+                        output.write('\t'.join(['d', chrom, str(delete), str(np.random.choice(range(4, 10))),np.random.choice(['homo', 'het'])])+'\n')
+            else:
+                with open(indel_file, 'w') as output:
+                    for i in range(self.in_no):
+                        chrom = np.random.choice(chroms.keys())
+                        chrom_size = chroms[chrom]
+                        insert = np.random.choice(range(1, chrom_size))
+                        output.write('\t'.join(['i', chrom, str(insert), self._generate_random_seq(),np.random.choice(['homo', 'het'])])+'\n')
+                    for j in range(self.del_no):
+                        chrom = np.random.choice(chroms.keys())
+                        chrom_size = chroms[chrom]
+                        delete = np.random.choice(range(1, chrom_size))
+                        output.write('\t'.join(['d', chrom, str(delete), str(np.random.choice(range(4, 10))),np.random.choice(['homo', 'het'])])+'\n')
+    
+    def _generate_reads_for_indel(self, params):
+        for sample in self.samples:
+            tasklogger.log_info('start generating reads for '+sample+'...')
+            fasta_file = os.path.join(params.out_dir, sample + '-ref.fasta')
+            # self.samples[sample]['fasta_file'] = fasta_file
+            command = 'scssim simuvars -r {0} -v {1} -o {2}'.format(params.ref_file, self.samples[sample]['indel_file'], fasta_file)
+            code = os.system(command)
+            if code != 0:
+                tasklogger.log_error('Errors happend when simulating varations with indel file...')
+            
+            #generate reads
+            out_prefix = os.path.join(params.out_dir, sample)
+            command = 'scssim genreads -i {0} -m {1} -l {2} -c {3} -s {4} -t {5} -o {6}'.format(fasta_file, params.profile_file, params.layout, params.coverage, params.isize, params.threads, out_prefix)
+            code = os.system(command)
+            if code != 0:
+                tasklogger.log_error('Errors happend when simulating reads...')
+            
+            # clean sample fasta file
+            os.remove(fasta_file)
+            os.remove(fasta_file+'.fai')
+    
+    # def _generate_reads_for_snp(self, params):
+    #     tasklogger.log_info('start genrating reads with fastq format...')
+    #     for sample in self.samples:
+    #         out_prefix = os.path.join(params.out_dir, sample)
+    #         command = 'scssim genreads -i {0} -m {1} -l {2} -c {3} -s {4} -t {5} -o {6}'.format(self.samples[sample]['fasta_file'], params.profile_file, params.layout, params.coverage, params.isize, params.threads, out_prefix)
+    #         code = os.system(command)
+    #         if code != 0:
+    #             tasklogger.log_error('Errors happend when simulating reads...')
+            
+    #         # clean sample fasta file
+    #         os.remove(self.samples[sample]['fasta_file'])
+
+    def sim_samples(self, params):
+        self._generate_indel_file(params)
+        # self._sim_fasta_for_snps(params)
+        self._generate_reads_for_indel(params)
+
+class SNVSimulator:
+    def __init__(self, cell_no=1, snv_no=100, verbose=1):
+        self.cell_no = cell_no
+        self.snv_no = snv_no
+        self._check_params()
+        self.samples = dict.fromkeys(['cell' + str(i+1) for i in range(self.cell_no)])
+        for sample in self.samples:
+            self.samples[sample] = {}
+        tasklogger.set_level(verbose)
+
+
+    def _check_params(self):
+        """Check SCSilicon parameters
+
+        This allows us to fail early - otherwise certain unacceptable
+        parameter choices, such as threads='10.5', would only fail after
+        minutes of runtime.
+
+        Raises
+        ------
+        ValueError : unacceptable choice of parameters
+        """
+        utils.check_int(cell_no=self.cell_no)
+        utils.check_positive(cell_no=self.cell_no)
+        utils.check_int(snv_no=self.snv_no)
+        utils.check_positive(snv_no=self.snv_no)
+
+    def set_params(self, **params):
+        """Set the parameters of SCSilicon.
+
+        Any parameters not given as named arguments will be left at their
+        current value.
+
+        Parameters
+        ----------
+
+        cell_no : int, optional, default: 1
+            The sample number for this simulation
+        
+        snv_no : int, optional, default: 1000
+            The SNV number of each sample
+        
+        verbose : `int` or `boolean`, optional, default: 1
+            If `True` or `> 0`, print log messages
+
+        Returns
+        -------
+        self
+        """
+
+        # parameters
+        if 'cell_no' in params and params['cell_no'] != self.cell_no:
+            self.cell_no = params['cell_no']
+            del params['cell_no']
+        if 'snv_no' in params and params['snv_no'] != self.snv_no:
+            self.snv_no = params['snv_no']
+            del params['snv_no']
+        if 'verbose' in params:
+            self.verbose = params['verbose']
+            tasklogger.set_level(self.verbose)
+            del params['verbose']
+        self._check_params()
+        self.get_params()
+        return self
+
+    def get_params(self):
+        print(vars(self))
+
+    def _generate_snv_file(self, params):
+        tasklogger.log_info('start generating snv file...')
+        chrom_sizes = pd.read_csv(params.chrom_size_file, header=None, sep='\t')
+        if params.chrom != 'all':
+            chrom = params.chrom
+            chrom_size = int(chrom_sizes[chrom_sizes[0]==chrom][1])
+        else:
+            chroms = {}
+            for index, row in chrom_sizes.iterrows():
+                [chrom, chrom_size] = row
+                chroms[chrom] = chrom_size
+
+        #out indel file
+        for sample in self.samples:
+            snv_file =os.path.join(params.out_dir, sample + '-snv.txt')
+            self.samples[sample]['snv_file'] = snv_file
+            
+            if params.chrom != 'all':
+                snvs = np.sort(np.random.choice(range(1, chrom_size), self.snv_no, replace=False))
+                with open(snv_file, 'w') as output:
+                    for snv in snvs:
+                        alleles = np.random.choice(['a','t','g','c'],2, replace=False)
+                        output.write('\t'.join(['s', chrom, str(snv), alleles[0], alleles[1], np.random.choice(['homo', 'het'])])+'\n')
+            else:
+                with open(snv_file, 'w') as output:
+                    for i in range(self.snv_no):
+                        chrom = np.random.choice(chroms.keys())
+                        chrom_size = chroms[chrom]
+                        snv = np.random.choice(range(1, chrom_size))
+                        alleles = np.random.choice(['a','t','g','c'],2, replace=False)
+                        output.write('\t'.join(['s', chrom, str(snv), alleles[0], alleles[1], np.random.choice(['homo', 'het'])])+'\n')
+    
+    def _generate_reads_for_snv(self, params):
+        for sample in self.samples:
+            tasklogger.log_info('start generating reads for '+sample+'...')
+            fasta_file = os.path.join(params.out_dir, sample + '-ref.fasta')
+            # self.samples[sample]['fasta_file'] = fasta_file
+            command = 'scssim simuvars -r {0} -v {1} -o {2}'.format(params.ref_file, self.samples[sample]['snv_file'], fasta_file)
+            code = os.system(command)
+            if code != 0:
+                tasklogger.log_error('Errors happend when simulating varations with snv file...')
+            
+            #generate reads
+            out_prefix = os.path.join(params.out_dir, sample)
+            command = 'scssim genreads -i {0} -m {1} -l {2} -c {3} -s {4} -t {5} -o {6}'.format(fasta_file, params.profile_file, params.layout, params.coverage, params.isize, params.threads, out_prefix)
+            code = os.system(command)
+            if code != 0:
+                tasklogger.log_error('Errors happend when simulating reads...')
+            
+            # clean sample fasta file
+            os.remove(fasta_file)
+            os.remove(fasta_file+'.fai')
+    
+    # def _generate_reads_for_snp(self, params):
+    #     tasklogger.log_info('start genrating reads with fastq format...')
+    #     for sample in self.samples:
+    #         out_prefix = os.path.join(params.out_dir, sample)
+    #         command = 'scssim genreads -i {0} -m {1} -l {2} -c {3} -s {4} -t {5} -o {6}'.format(self.samples[sample]['fasta_file'], params.profile_file, params.layout, params.coverage, params.isize, params.threads, out_prefix)
+    #         code = os.system(command)
+    #         if code != 0:
+    #             tasklogger.log_error('Errors happend when simulating reads...')
+            
+    #         # clean sample fasta file
+    #         os.remove(self.samples[sample]['fasta_file'])
+
+    def sim_samples(self, params):
+        self._generate_snv_file(params)
+        # self._sim_fasta_for_snps(params)
+        self._generate_reads_for_snv(params)
